'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var helpers = require('./helpers.js');
var render = require('./render.js');
var mergeSlots = require('./mergeSlots.js');

const define = (name, factory, template, options = {}) =>
  customElements.define(
    name,
    class extends HTMLElement {
      async connectedCallback() {
        if (!this.initialised) {
          this.$viewmodel = factory(this);

          if (this.$viewmodel instanceof Promise)
            this.$viewmodel = await this.$viewmodel;

          let observedProps = Object.keys(this.$viewmodel).filter(
            (v) => v.charAt(0) === "$"
          );

          let observedAttributes = observedProps
            .map((v) => v.slice(1))
            .map(helpers.pascalToKebab);

          let sa = this.setAttribute;
          this.setAttribute = (k, v) => {
            if (observedAttributes.includes(k)) {
              let { name, value } = helpers.attributeToProp(k, v);
              this.$viewmodel["$" + name] = value;
            }
            sa.apply(this, [k, v]);
          };

          observedAttributes.forEach((name) => {
            let property = helpers.attributeToProp(name).name;

            let value;

            if (this.hasAttribute(name)) {
              value = this.getAttribute(name);
            } else {
              value = this[property] || this.$viewmodel["$" + property];
            }

            Object.defineProperty(this, property, {
              get() {
                return this.$viewmodel["$" + property];
              },
              set(v) {
                this.$viewmodel["$" + property] = v;
                if (helpers.isPrimitive(v)) {
                  helpers.applyAttribute(this, property, v);
                }
              },
            });

            this[property] = value;
          });

          let beforeMountCallback;

          if (options.shadow) {
            this.attachShadow({
              mode: options.shadow,
            });
          } else {
            beforeMountCallback = (frag) => mergeSlots.mergeSlots(this, frag);
          }

          this.$viewmodel = render.render(
            this.shadowRoot || this,
            this.$viewmodel,
            template,
            () => {
              observedProps.forEach((k) => {
                let v = this.$viewmodel[k];
                if (helpers.isPrimitive(v)) helpers.applyAttribute(this, k.slice(1), v);
              });
            },
            beforeMountCallback
          );
          this.initialised = true;
        }
        this.$viewmodel.connectedCallback?.(this.$viewmodel);
      }
      disconnectedCallback() {
        this.$viewmodel?.disconnectedCallback?.(this.$viewmodel);
      }
    }
  );

exports.define = define;
